"""Module for testing AAKR."""


import pytest

from sklearn.datasets import load_linnerud
from sklearn.utils.testing import assert_allclose

from aakr import AAKR


@pytest.fixture
def data():
    return load_linnerud(return_X_y=True)


def test_aakr(data):
    X = data[0]
    aakr = AAKR()
    assert aakr.metric == 'euclidean'
    assert aakr.bw == 1
    assert aakr.n_jobs is None

    aakr.fit(X)
    assert hasattr(aakr, 'X_')

    X_nc = aakr.predict(X[:3])
    assert_allclose(X_nc, X[:3])


def test_aakr_input_shape_mismatch(data):
    X = data[0]
    aakr = AAKR().fit(X)
    assert aakr.X_.shape[1] == X.shape[1]

    with pytest.raises(ValueError, match='Shape of input is different'):
        aakr.predict(X[:3, :-1])



import setuptools


with open('README.md', 'r') as f:
    long_description = f.read()

with open('VERSION', 'r') as f:
    version = f.read().strip()

setuptools.setup(
    name='aakr',
    version=version,
    license='MIT',
    description='Implementation of Auto Associative Kernel Regression (AAKR)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Jesse Myrberg',
    author_email='jesse.myrberg@gmail.com',
    url='https://github.com/jmyrberg/aakr',
    keywords=['aakr', 'auto', 'associative', 'kernel', 'regression', 'anomaly', 'detection'],
    install_requires=[
        'numpy>=1.19.4',
        'pandas>=1.1.5',
        'scikit-learn>=0.23.2'
    ],
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS'
    ],
    extras_require={
        'tests': [
            'pytest',
            'pytest-cov'],
        'docs': [
            'sphinx',
            'sphinx-gallery',
            'sphinx_rtd_theme',
            'numpydoc',
            'matplotlib'
        ]
    }
)



"""Module for models."""


import numpy as np

from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.metrics.pairwise import pairwise_distances
from sklearn.utils.validation import check_array, check_is_fitted


class AAKR(TransformerMixin, BaseEstimator):
    """A template estimator to be used as a reference implementation.

    For more information regarding how to build your own estimator, read more
    in the :ref:`User Guide <user_guide>`.

    Parameters
    ----------
    demo_param : str, default='demo_param'
        A parameter used for demonstation of how to pass and store paramters.

    Examples
    --------
    >>> from aakr import AAKR
    >>> import numpy as np
    >>> X = np.arange(100).reshape(50, 2)
    >>> aakr = AAKR()
    >>> aakr.fit(X)
    AAKR(metric='euclidean', bw=1, n_jobs=None)
    """
    def __init__(self, metric='euclidean', bw=1, n_jobs=None):
        self.metric = metric
        self.bw = bw
        self.n_jobs = n_jobs

    def fit(self, X, y=None):
        """A reference implementation of a fitting function for a transformer.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            Training examples from normal conditions.
        y : None
            Not needed, exists for compability purposes.

        Returns
        -------
        self : object
            Returns self.
        """
        X = check_array(X)
        self.X_ = X
        return self

    def predict(self, X, **kwargs):
        """ A reference implementation of a prediction for a classifier.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            The input samples.

        Returns
        -------
        X_nc : ndarray, shape (n_samples, n_features)
            Expected values in normal conditions for each sample and feature.
        """
        # Validation
        check_is_fitted(self, 'X_')

        X = check_array(X)

        if X.shape[1] != self.X_.shape[1]:
            raise ValueError('Shape of input is different from what was seen'
                             'in `fit`')

        # Kernel regression
        D = pairwise_distances(X=self.X_, Y=X, metric=self.metric,
                               n_jobs=self.n_jobs, **kwargs)
        k = 1 / np.sqrt(2 * np.pi * self.bw ** 2)
        w = k * np.exp(-D ** 2 / (2 * self.bw ** 2))
        X_nc = w.T.dot(self.X_) / w.sum(0)[:, None]

        return X_nc
